#!/usr/bin/python

################################################################################
#
# A bunch of regression type tests for the builder and parser.
#
################################################################################

from SOAP import *
import sys


x = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<soap:Body soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SomeMethod>
<Result>
<Book>
   <title>My Life and Work</title>
</Book>
<Person>
   <name>Henry Ford</name>
   <age> 49 </age>
   <height> 5.5 </height>
</Person>
</Result>
</SomeMethod>
</soap:Body>
</soap:Envelope>
'''

def negfloat(x):
    return float(x) * -1.0
# parse rules
pr = {'Result':
      {'Book':   {'title':"string"},
       'Person': {'age':"int",
                  'height':negfloat}
       }
      }
y = parseSOAPRPC(x, parse_rules=pr)
assert y.Result.Person.age == 49

assert y.Result.Person.height == -5.5

# Try the reverse

class Book(Struct):
    def __init__(self):
        self.title = "Title of a book"
class Person(Struct):
    def __init__(self):
        self.age = "49"
        self.height = "5.5"
class Result(Struct):
    def __init__(self):
        self._name = "Result"
        self.Book = Book()
        self.Person = Person()

x = buildSOAP(Result())
#sys.exit(0)

x='''<SOAP-ENV:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.microsoft.com/soap/encoding/clr/1.0 http://schemas.xmlsoap.org/soap/encoding/" xmlns:i3="http://soapinterop.org/xsd" xmlns:i2="http://soapinterop.org/">
<SOAP-ENV:Body>
<i2:echoStructArray id="ref-1">
<return href="#ref-4"/>
</i2:echoStructArray>
<SOAP-ENC:Array id="ref-4" SOAP-ENC:arrayType="i3:SOAPStruct[3]">
<item href="#ref-5"/>
<item href="#ref-6"/>
<item href="#ref-7"/>
</SOAP-ENC:Array>
<i3:SOAPStruct id="ref-5">
<varString xsi:type="xsd:string">West Virginia</varString>
<varInt xsi:type="xsd:int">-546</varInt>
<varFloat xsi:type="xsd:float">-5.398</varFloat>
</i3:SOAPStruct>
<i3:SOAPStruct id="ref-6">
<varString xsi:type="xsd:string">New Mexico</varString>
<varInt xsi:type="xsd:int">-641</varInt>
<varFloat xsi:type="xsd:float">-9.351</varFloat>
</i3:SOAPStruct>
<i3:SOAPStruct id="ref-7">
<varString xsi:type="xsd:string">Missouri</varString>
<varInt xsi:type="xsd:int">-819</varInt>
<varFloat xsi:type="xsd:float">1.495</varFloat>
</i3:SOAPStruct>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

y = parseSOAPRPC(x)
assert getattr(y,"return")[0].varInt == -546
assert getattr(y,"return")[1].varInt == -641
assert getattr(y,"return")[2].varInt == -819
#sys.exit(0)
x = '''<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
<SOAP-ENV:Body SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsd2="http://www.w3.org/2000/10/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance" xmlns:xsi2="http://www.w3.org/2000/10/XMLSchema-instance">
<ns0:echoStringArrayResponse xmlns:ns0="http://soapinterop.org/">
<return2 href="#id3"/>
</ns0:echoStringArrayResponse>
<a id="id0" xmlns:ns0="http://soapinterop.org/" xsi2:type="xsd:string" xsi:type="xsd:string"></a>
<a id="id1" xmlns:ns0="http://soapinterop.org/" xsi2:type="xsd:string" xsi:type="xsd:string">Hello</a>
<a id="id2" xmlns:ns0="http://soapinterop.org/" xsi2:type="xsd:string" xsi:type="xsd:string">\'&lt;&amp;&gt;&quot;</a>
<return2 SOAP-ENC:arrayType="xsd:string[3]" id="id3" xmlns:ns0="http://soapinterop.org/">
<a href="#id0"/>
<a href="#id1"/>
<a href="#id2"/>
</return2>
</SOAP-ENV:Body></SOAP-ENV:Envelope>'''

y = parseSOAPRPC(x)
assert y.return2[1] == "Hello"

x = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<soap:Body soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SomeMethod>
<Result>
<Book>
   <title>My Life and Work</title>
   <author href="#Person-1"/>
</Book>
<Person id="Person-1">
   <name>Henry Ford</name>
   <address href="#Address-2"/>
</Person>
<Address id="Address-2">
   <email>mailto:henryford@hotmail.com</email>
   <web>http://www.henryford.com</web>
   <pers href="#Person-1"/>
</Address>
</Result>
</SomeMethod>
</soap:Body>
</soap:Envelope>
'''
y = parseSOAPRPC(x)
assert y.Result.Book.author.name == "Henry Ford"
assert y.Result.Book.author.address.web == "http://www.henryford.com"
assert y.Result.Book.author.address.pers.name == "Henry Ford"


# ref example
x = '''<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/" xmlns:soapenc="http://schemas.xmlsoap.org/soap/encoding/" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema">
<soap:Body soap:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<echoFloatArrayResponse xmlns="http://soapinterop.org/">
<Return href="#1" xmlns="" />
</echoFloatArrayResponse>
<soapenc:Array id="1" soapenc:arrayType="xsd:float[4]">
<Item>0</Item>
<Item>1</Item>
<Item>-1</Item>
<Item>3853.33325</Item>
</soapenc:Array>
</soap:Body>
</soap:Envelope>
'''

y = parseSOAPRPC(x)

assert y.Return[0] == 0
assert y.Return[1] == 1
assert y.Return[2] == -1



# Big terrible ordered data with attributes test.
data = '''<?xml version="1.0" encoding="UTF-8" ?>
<Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
<Body>
<replyBlock generic="1.0" attrib1="false" attrib2='hello'>
<itemList>
<mainItem mainattrib1='uno'>
<name>first_main_item</name>
<description>whatever etc.</description>
<infoList>
<itemInfo a1='123' a2='abc'>
<name>unoItem1</name>
</itemInfo>
<itemInfo a1='456' a2='def'>
<name>unoItem2</name>
</itemInfo>
<itemInfo a1='789' a2='ghi'>
<name>unoItem3</name>
</itemInfo>
</infoList>
</mainItem>
<mainItem mainattrib1='dos'>
<name>second_main_item</name>
<description>whatever etc.</description>
<infoList>
<itemInfo a1='3123' a2='3abc'>
<name>dosItem1</name>
</itemInfo>
<itemInfo a1='3456' a2='3def'>
<name>dosItem2</name>
</itemInfo>
<itemInfo a1='3789' a2='3ghi'>
<name>dosItem3</name>
</itemInfo>
</infoList>
</mainItem>
</itemList>
<itemList>
<mainItem mainattrib1='single'>
<name>single_main_item</name>
<description>whatever etc.</description>
<infoList>
<itemInfo a1='666' a2='xxx'>
<name>singleItem1</name>
</itemInfo>
</infoList>
</mainItem>
</itemList>
</replyBlock>
</Body>
</Envelope>'''

x = parseSOAP(data)
#print ".>",x.replyBlock.itemList._ns
y = buildSOAP(x)
#print y
#sys.exit(0)

my_xml2 = '''
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Header>
<t:Transaction xmlns:t="some-URI" SOAP-ENV:mustUnderstand="1">
5
</t:Transaction>
</SOAP-ENV:Header>
   <SOAP-ENV:Body>
       <m:GetLastTradePriceResponse xmlns:m="Some-URI">
            <PriceAndVolume>
                <LastTradePrice>
                    34.5
                </LastTradePrice>
                <DayVolume>
                    10000
                </DayVolume>
            </PriceAndVolume>
       </m:GetLastTradePriceResponse>
   </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

(x,h) = parseSOAPRPC(my_xml2,header=1)


x ='''
<V:Envelope
        xmlns:V="http://schemas.xmlsoap.org/soap/envelope/"
        xmlns:C="http://schemas.xmlsoap.org/soap/encoding/"
        xmlns:i="http://www.w3.org/1999/XMLSchema-instance"
        xmlns:d="http://www.w3.org/1999/XMLSchema"
        V:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<V:Body>
<m:echoStructArray
        xmlns:m="urn:xmethodsInterop">
<inputStructArray
        i:type="C:Array"
        C:arrayType="ns3:SOAPStruct[0]"
        xmlns:ns3="http://soapinterop.org/xsd"/>
</m:echoStructArray>
</V:Body>
</V:Envelope>'''

x = parseSOAPRPC(x)
#sys.exit(0)

x = '''<?xml version="1.0" encoding="UTF-8" standalone="no"?>

<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><m:echoStringResponse xmlns:m="http://soapinterop.org/"><Result id="fred">hello</Result></m:echoStringResponse></SOAP-ENV:Body></SOAP-ENV:Envelope>

'''

x = parseSOAPRPC(x)

assert x._getAttrs("Result")[(None, "id")] == "fred"



x='''<SOAP-ENV:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.microsoft.com/soap/encoding/clr/1.0 http://schemas.xmlsoap.org/soap/encoding/" xmlns:a1="http://schemas.microsoft.com/clr/ns/System.Runtime.Serialization.Formatters">

<SOAP-ENV:Body>

<SOAP-ENV:Fault id="ref-1">

<faultcode id="ref-2">SOAP-ENV:Server</faultcode>

<faultstring id="ref-3">Exception thrown on Server</faultstring>

<detail xsi:type="a1:ServerFault">


<exceptionType id="ref-4">System.Runtime.Serialization.SerializationException, mscorlib, Version=1.0.2411.0, Culture=neutral, PublicKeyToken=b77a5c561934e089</exceptionType>


<message id="ref-5">Soap Parser Error System.Runtime.Serialization.SerializationException: Parse Error, xsd type not valid: Array

   at System.Runtime.Serialization.Formatters.Soap.SoapHandler.ProcessGetType(String value, String xmlKey)

   at System.Runtime.Serialization.Formatters.Soap.SoapHandler.ProcessType(ParseRecord pr, ParseRecord objectPr)

   at System.Runtime.Serialization.Formatters.Soap.SoapHandler.ProcessAttributes(ParseRecord pr, ParseRecord objectPr)

   at System.Runtime.Serialization.Formatters.Soap.SoapHandler.StartElement(String prefix, String name, String urn)

   at System.XML.XmlParser.ParseElement()

   at System.XML.XmlParser.ParseTag()

   at System.XML.XmlParser.Parse()

   at System.XML.XmlParser.Parse0()

   at System.XML.XmlParser.Run()</message>

   
<stackTrace id="ref-6">   at System.Runtime.Serialization.Formatters.Soap.SoapHandler.Error(IXmlProcessor p, Exception ex)

   at System.XML.XmlParser.Run()

   at System.Runtime.Serialization.Formatters.Soap.SoapParser.Run()

   at System.Runtime.Serialization.Formatters.Soap.ObjectReader.Deserialize(HeaderHandler handler, ISerParser serParser)

   at System.Runtime.Serialization.Formatters.Soap.SoapFormatter.Deserialize(Stream serializationStream, HeaderHandler handler)

   at System.Runtime.Remoting.Channels.CoreChannel.DeserializeMessage(String mimeType, Stream xstm, Boolean methodRequest, IMessage msg, Header[] h)

   at System.Runtime.Remoting.Channels.SoapServerFormatterSink.ProcessMessage(IServerChannelSinkStack sinkStack, ITransportHeaders requestHeaders, Stream requestStream, IMessage&#38; msg, ITransportHeaders&#38; responseHeaders, Stream&#38; responseStream)</stackTrace>

</detail>


</SOAP-ENV:Fault>

</SOAP-ENV:Body>

</SOAP-ENV:Envelope>

'''

z = parseSOAPRPC(x)


x = '''<?xml version="1.0" encoding="UTF-8" standalone="no"?>

<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"><SOAP-ENV:Body><m:echoStringResponse xmlns:m="http://soapinterop.org/"><Result></Result></m:echoStringResponse></SOAP-ENV:Body></SOAP-ENV:Envelope>

'''

z = parseSOAPRPC(x)
assert type(z.Result) == type('')

x = [1,2,3,4,5]
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert x==z


x = ["cayce", "asd", "buy"]
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert x==z


my_xml = '''
<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
  <SOAP-ENV:Body>
    <m:getStateName xmlns:m="http://www.soapware.org/">
      <statenum xsi:type="xsd:int">41</statenum>
    </m:getStateName>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

s = parseSOAPRPC(my_xml)
assert s.statenum == 41
assert type(s.statenum) == type(0)


my_xml_ns = '''
<XSOAP-ENV:Envelope XSOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:XSOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:XSOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:Xxsd="http://www.w3.org/1999/XMLSchema" xmlns:Xxsi="http://www.w3.org/1999/XMLSchema-instance">
  <XSOAP-ENV:Body>
    <m:getStateName xmlns:m="http://www.soapware.org/">
      <statenum Xxsi:type="Xxsd:int">41</statenum>
    </m:getStateName>
  </XSOAP-ENV:Body>
</XSOAP-ENV:Envelope>
'''

s = parseSOAPRPC(my_xml_ns)
assert s.statenum == 41, "NS one failed"
assert type(s.statenum) == type(0)

my_xml2 = '''
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/">
<SOAP-ENV:Header>
<t:Transaction xmlns:t="some-URI" SOAP-ENV:mustUnderstand="1">
5
</t:Transaction>
</SOAP-ENV:Header>
   <SOAP-ENV:Body>
       <m:GetLastTradePriceResponse xmlns:m="Some-URI">
            <PriceAndVolume>
                <LastTradePrice>
                    34.5
                </LastTradePrice>
                <DayVolume>
                    10000
                </DayVolume>
            </PriceAndVolume>
       </m:GetLastTradePriceResponse>
   </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

s = parseSOAPRPC(my_xml2)
assert s.PriceAndVolume.LastTradePrice.strip() == "34.5"
assert s.PriceAndVolume.DayVolume.strip() == "10000"


my_xml3 = '''
<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
   <SOAP-ENV:Body>
   <Bounds>
   <param>
   <lowerBound xsi:type="xsd:int"> 18 </lowerBound>
   <upperBound xsi:type="xsd:int"> 139</upperBound>
   </param>
   </Bounds>
   </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

s = parseSOAPRPC(my_xml3)
assert s.param.lowerBound == 18
assert s.param.upperBound == 139


my_xml4 = '''
<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
   <SOAP-ENV:Body>
   <Bounds>
<param SOAP-ENC:arrayType="xsd:ur-type[4]" xsi:type="SOAP-ENC:Array"><item xsi:type="xsd:int">12</item>
   <item xsi:type="xsd:string">Egypt</item>
   <item xsi:type="xsd:boolean">0</item>
   <item xsi:type="xsd:int">-31</item>
   </param>
   <param1 xsi:null="1"></param1>
   </Bounds>
   </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

s = parseSOAPRPC(my_xml4)
assert s.param[0]==12
assert s.param[1]=="Egypt"
assert s.param[2]==0
assert s.param[3]==-31
assert s.param1==None


my_xml5 = '''
<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
   <SOAP-ENV:Body>
      <SOAP-ENV:Fault>
         <faultcode>SOAP-ENV:Client</faultcode>
         <faultstring>Cant call getStateName because there are too many parameters.</faultstring>
         </SOAP-ENV:Fault>
      </SOAP-ENV:Body>
   </SOAP-ENV:Envelope>
'''

s = parseSOAPRPC(my_xml5)
assert s.__class__ == Fault
assert s.faultcode == "SOAP-ENV:Client"

my_xml6 = '''
<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENV:Body>
<h SOAP-ENC:arrayType="xsd:ur-type[6]" xsi:type="SOAP-ENC:Array">
<item xsi:type="xsd:int">5</item>
<item xsi:type="xsd:int">3</item>
<item xsi:type="xsd:int">2</item>
<item xsi:type="xsd:string">monkey</item>
<item xsi:type="xsd:string">cay</item>
<item>
<cat xsi:type="xsd:string">hello</cat>
<ferret SOAP-ENC:arrayType="xsd:ur-type[6]" xsi:type="SOAP-ENC:Array">
<item xsi:type="xsd:int">5</item>
<item xsi:type="xsd:int">4</item>
<item xsi:type="xsd:int">3</item>
<item xsi:type="xsd:int">2</item>
<item xsi:type="xsd:int">1</item>
<item>
<cow xsi:type="xsd:string">moose</cow>
</item>
</ferret>
<monkey xsi:type="xsd:int">5</monkey>
</item>
</h>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''

q = parseSOAPRPC(my_xml6)
assert q[0] == 5
assert q[1] == 3
assert q[2] == 2
assert q[3] == 'monkey'
assert q[4] == 'cay'
x = q[5]
assert x.monkey == 5
assert x.cat == "hello"
assert x.ferret[0] == 5
assert x.ferret[3] == 2
assert x.ferret[5].cow == "moose"



################################################################################
# test builder and parser
################################################################################

# test array
x = Array("spam", [5,4,3,21])
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert x==z

# test struct
x = Struct("eggs")
x.test = 5
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert x.test==z.test
# test faults
x = Fault("ServerError","Howdy",[5,4,3,2,1])
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert x.faultcode == z.faultcode
assert x.faultstring == z.faultstring
assert x.detail == z.detail

# Test the recursion
class one:
    def __init__(self):
        self.str = "one"
class two:
    def __init__(self):
        self.str = "two"
        
o = one()
t = two()
o.t = t
t.o = o
class three:
    def __init__(self):
        self.str = "three"
tre = three()
tre.o = o
tre.t = t
blewup = 0
#x = buildSOAP(Struct(data={'tre':tre}))
x = buildSOAP(tre)
y = parseSOAPRPC(x)
assert y.t.o.t.o.t.o.t.str == "two"




# Test the recursion with structs
        
o = Struct("one")
t = Struct("two")
o.t = t
o.str = "one"
t.o = o
t.str = "two"
tre = Struct("three")
tre.o = o
tre.t = t
tre.str = "three"
blewup = 0
#x = buildSOAP(Struct(data={'tre':tre}))
x = buildSOAP(tre)
y = parseSOAPRPC(x)
assert y.t.o.t.o.t.o.t.str == "two"





# test Boolean
x = Boolean(1)
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert int(x) == int(z)

# test dateTime
for t in (928182000, 928200900, 928163100, 928182000.2, 1000000.23432):
    d = DateTime (t)
    s = buildSOAP (d)
    u = parseSOAPRPC(s)

    if t != u:
        print "dateTime", t, "failed, SOAP =\n", s, "\nparsed value =", u

# test base64
m = "Test Message"
x = Binary(m)
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert m == z

m = "Test Message <tag> & </tag>"
x = Struct("test")
x.msg = m
y = buildSOAP(x)
z = parseSOAPRPC(y)
assert m == z.msg


my_xml7 = '''
<SOAP-ENV:Envelope SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENC="http://schemas.xmlsoap.org/soap/encoding/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:xsd="http://www.w3.org/1999/XMLSchema" xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
<SOAP-ENV:Body>
<Bounds>
<param>
<lowerBound xsi:type="xsd:int"> 18 </lowerBound>
<upperBound xsi:type="xsd:int"> 139</upperBound>
</param>
</Bounds>
</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
'''
x = parseSOAPRPC(my_xml7)
y = buildSOAP(x)

# Some encoding tests

try:
    # Does buildSOAP require a valid encoding?
    x = buildSOAP('hello', encoding = 'gleck')
except LookupError, e:
    if str (e) != 'unknown encoding': raise
    x = None
except:
    print "Got unexpected exception: %s %s" % tuple (sys.exc_info ()[0:2])
    x = ''
assert x == None

try:
    # Does SOAPProxy require a valid encoding?
    x = SOAPProxy('', encoding = 'gleck')
except LookupError, e:
    if str (e) != 'unknown encoding': raise
    x = None
except:
    print "Got unexpected exception: %s %s" % tuple (sys.exc_info ()[0:2])
    x = ''
assert x == None

try:
    # Does SOAPServer require a valid encoding?
    x = SOAPServer(('localhost', 0), encoding = 'gleck')
except LookupError, e:
    if str (e) != 'unknown encoding': raise
    x = None
except:
    print "Got unexpected exception: %s %s" % tuple (sys.exc_info ()[0:2])
    x = ''
assert x == None


encodings = ('US-ASCII', 'ISO-8859-1', 'UTF-8', 'UTF-16')

tests = ('A', u'\u0041')
for t in tests:
    for i in range (len (encodings)):
        x = buildSOAP (t, encoding = encodings[i])
        y = parseSOAPRPC (x)
        assert y == t

tests = (u'\u00a1',)
for t in tests:
    for i in range (len (encodings)):
        try:
            x = buildSOAP (t, encoding = encodings[i])
        except:
            if i > 0: raise
            continue
        y = parseSOAPRPC (x)
        assert y == t

tests = (u'\u01a1', u'\u2342')
for t in tests:
    for i in range (len (encodings)):
        try:
            x = buildSOAP (t, encoding = encodings[i])
        except:
            if i > 1: raise
            continue
        y = parseSOAPRPC (x)
        assert y == t

# Now some data type tests

# Make sure the various limits are checked when parsing

xml = '''<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope
    SOAP-ENV:encodingStyle="http://schemas.xmlsoap.org/soap/encoding/"
    xmlns:xsd="http://www.w3.org/1999/XMLSchema"
    xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/"
    xmlns:xsi="http://www.w3.org/1999/XMLSchema-instance">
    <SOAP-ENV:Body>
        <v1 xsi:type="xsd:%s">%s</v1>
    </SOAP-ENV:Body>
</SOAP-ENV:Envelope>'''

# First, integers

for t, l in SOAPParser.intlimits.items():
    try:
        parseSOAP(xml % (t, 'hello'))
        raise AssertionError, "parsed %s of 'hello' without error" % t
    except AssertionError:
        raise
    except:
        pass

    if l[0] != None:
        try:
            parseSOAP(xml % (t, l[0] - 1))
            raise AssertionError, "parsed %s of %s without error" % i
        except AssertionError:
            raise
        except UnderflowError:
            pass

    if l[1] != None:
        try:
            parseSOAP(xml % (t, l[1] + 1))
            raise AssertionError, "parsed %s of %s without error" % i
        except AssertionError:
            raise
        except OverflowError:
            pass

# Next, floats. Note that chances are good this won't work in any non-Unix
# Pythons.

import struct

for i in \
    (
        ('float', '3.5e38'),
        ('float', '6.9e-46'),
        ('double', '1.8e308'),
        ('double', '2.4e-324'),
    ):
    try:
        parseSOAP(xml % (i[0], i[1]))
        raise AssertionError, "parsed %s of %s without error" % i
    except AssertionError:
        raise
    except (UnderflowError, OverflowError):
        pass

# Make sure we can't instantiate the base class

try:
    x = urType()
    blowedup = 0
except:
    blowedup = 1

if blowedup != 1:
    raise AssertionError, "instantiated urType directly"

# boolean

# First some things that shouldn't be taken as boolean values

test = (10, 'hello', (), [])
for i in test:
    try:
        booleanType(i)
        raise AssertionError, \
            "instantiated a boolean with a bad value (%s)" % repr(i)
    except AssertionError:
        raise
    except:
        pass

# Now some things that should.

for i in ((0, 'false'), ('false', 'false'), (1, 'true'),
    ('true', 'true'), (0.0, 'false'), (1.0, 'true')):
    x = booleanType(i[0])

    if str(x) != i[1]:
        raise AssertionError, "expected %s, got %s" % (i[0], str(x))

# Can we give it a name?

x = booleanType(1, name = 'George', typed = 0)
y = buildSOAP(x)
z = parseSOAP(y)

test = 'true'
if z.George != test:
    raise AssertionError, "expected %s, got %s" % (repr(test), repr(z))

# base64Binary

x = ''
for i in range(256):
    x += chr(i)
x = (base64BinaryType(''), base64BinaryType(x),
    base64BinaryType('hello'))

# duration

# First some things that shouldn't be valid durations

test = ('hello', ('hello',), (-10, -10), (-10, 0, -10), (10.5, 10.5),
    (0, 10.5, 0, 10.5, 0), (1, 2, 3, 4, 5, 6, 7))
for i in test:
    try:
        durationType(i)
        raise AssertionError, \
            "instantiated a duration with a bad value (%s)" % repr(i)
    except AssertionError:
        raise
    except:
        pass

# Now some things that should

for i in \
    (
        (0, 'P0S'),
        ((), 'P0S'),
        ([], 'P0S'),
        (10, 'PT10S'),
        (-10, '-PT10S'),
        (10.5, 'PT10.5S'),
        ((10, 20), 'PT10M20S'),
        ((-10, 20), '-PT10M20S'),
        ((10, 0), 'PT10M'),
        ((10, 0, 0), 'PT10H'),
        ((10, 0, 0, 0), 'P10D'),
        ((10, 0, 0, 0, 0), 'P10M'),
        ((10, 0, 0, 0, 0, 0), 'P10Y'),
        ((-10, 0, 0, 0, 0, 0), '-P10Y'),
        ((10, 0, 0, 0, 0, 20), 'P10YT20S'),
        ((1, 2, 3, 4, 5, 6.7), 'P1Y2M3DT4H5M6.7S'),
        ((1e6, 2e6, 3e6, 4e6, 5e6, 6.7e6),
            'P1000000Y2000000M3000000DT4000000H5000000M6700000S'),
    ):
    x = durationType(i[0])

    if str(x) != i[1]:
        raise AssertionError, "for %s, expected %s, got %s" % \
            (repr(i[0]), i[1], str(x))

# dateTime

# First some things that shouldn't be valid dateTimes

test = ('hello', ('hello',), (1, 2, 3, 4, 5),
    (1, 2, 3, 4, 5, 6, 7, 8, 9, 10), (1, 2, 3, 4, 5, 'hello'))
for i in test:
    try:
        dateTimeType(i)
        raise AssertionError, \
            "instantiated a dateTime with a bad value (%s)" % repr(i)
    except AssertionError:
        raise
    except:
        pass

# Now some things that should

for i in \
    (
        (1, '1970-01-01T00:00:01Z'),
        (1.5, '1970-01-01T00:00:01.5Z'),
        (1e9, '2001-09-09T01:46:40Z'),
        ((1970, 1, 1, 0, 0, 0), '1970-01-01T00:00:00Z'),
        ((1990, 1, 1, 0, 0, 0, 0, 0, 0), '1990-01-01T00:00:00Z'),
        ((1990, 1, 1, 0, 0, 0.25, 0, 0, 0), '1990-01-01T00:00:00.25Z'),
        ((11990, 1, 1, 0, 0, 0.25, 0, 0, 0), '11990-01-01T00:00:00.25Z'),
    ):
    x = dateTimeType(i[0])

    if str(x) != i[1]:
        raise AssertionError, "for %s, expected %s, got %s" % \
            (repr(i[0]), i[1], str(x))

# time

# First some things that shouldn't be valid times

test = ('hello', ('hello',), (1, 2, 3, 4, 5),
    (1, 2, 3, 4, 5, 6, 7, 8), (1, 2, 'hello'), (25, 0, 0), (0, 0, 86400))
for i in test:
    try:
        timeType(i)
        raise AssertionError, \
            "instantiated a time with a bad value (%s)" % repr(i)
    except AssertionError:
        raise
    except:
        pass

# Now some things that should

for i in \
    (
        (1, '00:00:01Z'),
        (1.5, '00:00:01.5Z'),
        (86399.75, '23:59:59.75Z'),
        ((10, 20, 30), '10:20:30Z'),
        ((0, 0, 3661.5), '01:01:01.5Z'),
        ((10.5, 20, 30), '10:50:30Z'),
        ((10.75, 20, 30), '11:05:30Z'),
        ((10.236, 20, 30), '10:34:39.6Z'),
    ):
    x = timeType(i[0])

    if str(x) != i[1]:
        raise AssertionError, "for %s, expected %s, got %s" % \
            (repr(i[0]), i[1], str(x))

# date

# First some things that shouldn't be valid dates

test = ('hello', ('hello',), (1, 2, 3, 4, 5),
    (1, 2, 3, 4, 5, 6, 7, 8, 9, 10), (1, 2, 3, 4, 5, 'hello'))
for i in test:
    try:
        dateType(i)
        raise AssertionError, \
            "instantiated a date with a bad value (%s)" % repr(i)
    except AssertionError:
        raise
    except:
        pass

# Now some things that should

for i in \
    (
        (1, '1970-01-01Z'),
        (1.5, '1970-01-01Z'),
        (1e9, '2001-09-09Z'),
        ((1970, 1, 1), '1970-01-01Z'),
        ((1990, 1, 1, 0, 0, 0, 0, 0, 0), '1990-01-01Z'),
        ((1990, 1, 1, 0, 0, 0.25, 0, 0, 0), '1990-01-01Z'),
        ((11990, 1, 1, 0, 0, 0.25, 0, 0, 0), '11990-01-01Z'),
    ):
    x = dateType(i[0])

    if str(x) != i[1]:
        raise AssertionError, "for %s, expected %s, got %s" % \
            (repr(i[0]), i[1], str(x))


print "TESTS WERE SUCCESSFUL"
